import React from 'react';
import PropTypes from 'prop-types';

import amapConfig from './config/amap';
import gisModules from './config/module';

import AMapTileLayer from './extensions/layers/AMapTileLayer';
import GoogleTileLayer from './extensions/layers/GoogleTileLayer';

import * as Utils from './utils/Utils';

import * as GeomUtils from './utils/GeometryTool';

export default class MapManager extends React.Component {
  static defaultProps = {
    id: '#jsmap',
    width: '100%',
    height: '420px',
    mapsettings: {},
    modules: gisModules,
  };

  constructor(props) {
    super(props);
    this.mapRef = React.createRef();
    this.initializeVariables();
  }

  async initializeVariables() {
    this.view = null;
    this.map = null;
    this.basemap = null;
    this.tipManager = null;
    this.AreaLayerSum = null;
    this.reorderGroups = [];
    this.hidetipHandler = null;
    this.showtipHandler = null;
    this.renderMenu = true;
    this.mapsettings = this.props.mapsettings;
  }

  getStyle() {
    return {
      width: this.props.width,
      height: this.props.height,
    };
  }

  async componentDidMount() {
    let moduleArr = await Utils.asyncLoadModules(this.props.modules);
    await GeomUtils.loadAllDependencies();
    this.getModules(moduleArr);
    this.createMapLayers();
    this.renderView();
    this.loadAMap();
  }

  loadAMap() {
    console.log('load AMap');
    let url = 'https://webapi.amap.com/maps?v=1.4.15&key=e83f64300a2a55a33fa8e4ab9a46bca6';
    fetch(url).then((params) => {
      console.log(params);
      console.log(window);
      // let plugins = ['AMap.Autocomplete', 'AMap.PlaceSearch', 'AMap.DistrictSearch']
      // AMap.plugins(plugins, () => {

      // })
      let UIurl = '//webapi.amap.com/ui/1.1/main.js';
      fetch(UIurl, () => {});
    });
  }

  getModules(moduleArr) {
    this.loadedModules = {
      Map: moduleArr[0],
      BaseMap: moduleArr[1],
      MapView: moduleArr[2],
      SceneView: moduleArr[3],
      TileLayer: moduleArr[4],
      MapImageLayer: moduleArr[5],
      ElevationLayer: moduleArr[6],
      Ground: moduleArr[7],
      SceneLayer: moduleArr[8],
      FeatureLayer: moduleArr[9],
      GroupLayer: moduleArr[10],
      VectorTileLayer: moduleArr[11],
      OpenStreetMapLayer: moduleArr[12],
      Extent: moduleArr[13],
      Collection: moduleArr[14],
      Camera: moduleArr[15],
      Viewpoint: moduleArr[16],
      jsonUtils: moduleArr[17],
      watchUtils: moduleArr[18],
      Graphic: moduleArr[19],
      PictureMarkerSymbol: moduleArr[20],
      Popup: moduleArr[21],
      GraphicsLayer: moduleArr[22],
      matrix: moduleArr[23],
      Color: moduleArr[24],
      SimpleFillSymbol: moduleArr[25],
      SimpleLineSymbol: moduleArr[26],
      TextSymbol: moduleArr[27],
      Point: moduleArr[28],
      lang: moduleArr[29],
      promiseUtils: moduleArr[30],
      urlUtils: moduleArr[31],
    };
    Utils.bindUrlUtils(this.loadedModules.urlUtils);
  }

  createMapLayers() {
    this.loadedModules.AMapTileLayer = AMapTileLayer(
      this.loadedModules.TileLayer,
      this.loadedModules.lang,
      this.loadedModules.promiseUtils,
    );
    this.loadedModules.GoogleTileLayer = GoogleTileLayer(
      this.loadedModules.TileLayer,
      this.loadedModules.lang,
      this.loadedModules.promiseUtils,
    );
  }

  renderView() {
    this.addLayerProxy();
    this.addMap();
    this.addBasemaps();
    this.addLayerReorder();
    this.addLayers();
    if ('2d' == this.mapsettings.type.toLowerCase()) {
      this.addMatrixLayer(); //加入空的GraphicsLayer，用于获取地图的moveStart,viewChange,moveEnd事件
    }
    this.addTipManager();
    if (this.renderMenu && this.view && this.view.type == '2d') {
      this.loadedModules.watchUtils.whenTrue(
        this.view,
        'ready',
        this.renderRightClickMenu.bind(this),
      );
      this.loadedModules.watchUtils.whenTrueOnce(this.view, 'ready', this.renderRoamte.bind(this));
      // edit by zhangyao on 2018/11/1
      this.hidetipHandler = this.loadedModules.watchUtils.whenTrue(
        this.view,
        'stationary',
        function (e) {
          console.log('地图禁止');
          // this.tipManager.$el.show();
          // this.$el.find(".gis-view-surface div.gis-display-object").show();
          if (this.updateLayerView) this.updateLayerView();
        }.bind(this),
      );
      this.showtipHandler = this.loadedModules.watchUtils.whenFalse(
        this.view,
        'stationary',
        function (e) {
          if (this.view.animation) {
            // 判断地图是否在缩放
            // this.tipManager.$el.hide();
            // this.$el.find(".gis-view-surface div.gis-display-object").hide();
            console.log('地图移动');
          }
        }.bind(this),
      );
    }
  }

  addLayerProxy() {}

  addMap() {
    const mapsettings = this.mapsettings;
    this.map = new this.loadedModules.Map();
    if (this.mapsettings.ground) {
      if ('string' == typeof this.mapsettings.ground) {
        this.map.ground = this.mapsettings.ground;
      } else {
        var layers = mapsettings.ground.layers,
          layer,
          groundlayers = new this.loadedModules.Collection();
        layers.forEach(
          function (layerconfig) {
            this.createLayer(layerconfig, groundlayers, false, '');
          }.bind(this),
        );
        this.map.ground = new Ground({
          layers: groundlayers,
        });
      }
    }
    if ('2d' == mapsettings.type.toLowerCase()) {
      this.view = new this.loadedModules.MapView({
        map: this.map,
        container: this.mapRef.current,
        manager: this,
        resizeAlign: mapsettings.resizeAlign || 'top-left',
      });
    } else if ('3d' == mapsettings.type.toLowerCase()) {
      this.view = new this.loadedModules.SceneView({
        map: this.map,
        manager: this,
        qualityProfile: mapsettings.qualityProfile || 'high',
        container: this.mapRef.current,
      });
      if (mapsettings.viewingMode) {
        this.view.viewingMode = mapsettings.viewingMode;
      }
      if (mapsettings.environment) {
        this.view.environment = mapsettings.environment;
      }
      if (mapsettings.clippingArea) {
        this.view.clippingArea = new Extent(mapsettings.clippingArea);
      }
      if (mapsettings.extent) {
        this.view.extent = new Extent(mapsettings.extent);
      }
    }
    //设置View初始范围
    this.viewpoint = mapsettings.viewpoint;
    this.setInitialViewpoint = this.setInitialViewpoint.bind(this);
    this.view.then(this.setInitialViewpoint);
  }

  setInitialViewpoint() {
    window.view = this.view;
    if (this.viewpoint) {
      this.initialViewpoint = new this.loadedModules.Viewpoint();
      if (this.viewpoint.camera) {
        this.initialViewpoint.camera = new this.loadedModules.Camera(this.viewpoint.camera);
        this.viewpoint.targetGeometry &&
          (this.initialViewpoint.targetGeometry = GeomUtils.toGeometry(
            this.viewpoint.targetGeometry,
          ));
        this.viewpoint.scale && (this.initialViewpoint.scale = this.viewpoint.scale);
        this.viewpoint.rotation && (this.initialViewpoint.rotation = this.viewpoint.rotation);
      }
      var geom = this.initialViewpoint.targetGeometry;
      if (geom && !(geom.xmin == 0 && geom.ymin == 0 && geom.xmax == 0 && geom.ymax == 0)) {
        if (geom.type == 'extent') {
          var scale = this.getNearestScale(geom.clone());
          if (scale != -1) {
            this.initialViewpoint.scale = scale;
          }
        }
      } else {
        if (this.view.extent) {
          this.initialViewpoint.targetGeometry = this.view.extent.clone();
          this.initialViewpoint.scale = this.view.scale;
        }
      }
      this.view.goTo(this.initialViewpoint);
    } else {
      this.initialViewpoint = this.view.viewpoint;
    }

    if (
      this.mapsettings.areasettings &&
      this.mapsettings.areasettings.areaName.toLowerCase().indexOf('offlinearea') > -1 &&
      !this.areaLayerInit
    ) {
      this.extent = {};
      this.setExtent();
      this.areaLayerInit = true;
      return;
    }

    if (!this.areaLayerInit) {
      this.loadAreaLayer();
    }
  }

  addBasemaps() {
    let basemapconfigs = this.mapsettings.basemaps;
    this.basemaps = [];
    if (basemapconfigs) {
      basemapconfigs.forEach(
        function (basemapconfig) {
          this.basemaps.push(this.createBasemap(basemapconfig));
        }.bind(this),
      );
    } else {
      this.basemaps.push(this.createDefaultBasemap());
    }

    if (this.basemaps.length) {
      this.map.basemap = this.basemaps[0];
    }
  }

  createBasemap(basemap) {
    var bm;
    if (basemap) {
      if ('string' == typeof basemap) {
        bm = basemap;
      } else {
        // var layer,
        var bls = new this.loadedModules.Collection(),
          id = basemap.id,
          title = basemap.title,
          thumbnailUrl = basemap.thumbnailUrl,
          baseLayers = basemap.baseLayers;
        baseLayers.forEach(
          function (layerconfig) {
            this.createLayer(layerconfig, bls, false, 'baseMap');
          }.bind(this),
        );
        bm = new this.loadedModules.BaseMap({
          id: id,
          title: title,
          thumbnailUrl: thumbnailUrl,
          baseLayers: bls,
        });
      }
    }
    return bm;
  }

  createDefaultBasemap() {}

  addLayerReorder() {
    this.layerAddedHandler = this.layerAddedHandler.bind(this);
    this.map.layers.on('after-add', this.layerAddedHandler);
  }

  layerAddedHandler(event) {
    var lyr = event.item;
    if ('graphics' === lyr.type) {
      if (this.isInGroup(lyr)) {
        var layer = this.map.layers.find(
          function (l) {
            return this.getLayerWeight(l) > this.getLayerWeight(lyr);
          }.bind(this),
        );
        if (layer) {
          this.map.layers.reorder(lyr, this.map.layers.indexOf(layer) - 1);
        }
      }
    }
  }

  isInGroup(lyr) {
    var groupName = lyr.order.split('-')[0],
      lyrIndex = Number(lyr.order.split('-')[1]),
      groupIndex = this.reorderGroups.indexOf(groupName);
    return _.isNumber(lyrIndex) && groupIndex >= 0;
  }

  getLayerWeight() {
    if ('graphics' === lyr.type) {
      var groupName = lyr.order.split('-')[0],
        lyrIndex = Number(lyr.order.split('-')[1]),
        groupIndex = this.reorderGroups.indexOf(groupName);
      if (_.isNumber(lyrIndex) && groupIndex >= 0) {
        return (groupIndex + 1) * 100 + lyrIndex;
      } else {
        return 0;
      }
    } else {
      return -1;
    }
  }

  registerReorderStrategy(groupName) {
    var index = this.reorderGroups.indexOf(groupName);
    index < 0 && this.reorderGroups.push(groupName);
  }

  addLayers() {
    let layerconfigs = this.mapsettings.layers;
    if (layerconfigs) {
      var layer;
      layerconfigs.forEach(
        function (layerconfig) {
          if (!layerconfig.id) return;
          this.createLayer(layerconfig, this.map, false, 'pipeLayer');
        }.bind(this),
      );
    }
  }

  addMatrixLayer() {
    this._matrixLayer = new this.loadedModules.GraphicsLayer();
    this.view.map.add(this._matrixLayer);
    this.matrixLayerViewCreated = this.matrixLayerViewCreated.bind(this);
    this.matrixLayerMoveStart = this.matrixLayerMoveStart.bind(this);
    this.matrixLayerViewChange = this.matrixLayerViewChange.bind(this);
    this.matrixLayerMoveEnd = this.matrixLayerMoveEnd.bind(this);
    this.view.whenLayerView(this._matrixLayer).then(this.matrixLayerViewCreated);
  }

  matrixLayerViewCreated(lyrView) {
    if (lyrView) {
      lyrView.on('moveStart', this.matrixLayerMoveStart);
      lyrView.on('viewChange', this.matrixLayerViewChange);
      lyrView.on('moveEnd', this.matrixLayerMoveEnd);
    }
  }

  matrixLayerMoveStart(lyrView) {
    var m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveStart', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  }

  matrixLayerViewChange(lyrView) {
    var m = this.getLayerViewMatrix(lyrView);
    this.view.emit('viewChange', {
      layerView: lyrView,
      matrix: m,
      zooming: m.xx != 1,
      matrixString: this.toMatrixString(m),
    });
  }

  matrixLayerMoveEnd(lyrView) {
    var m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveEnd', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  }

  getLayerViewMatrix(lyrView) {
    let matrix = this.loadedModules.matrix;
    var m = matrix.clone(matrix.identify);
    if (
      lyrView.graphicsView &&
      lyrView.graphicsView._frontGroup &&
      lyrView.graphicsView._frontGroup.g &&
      lyrView.graphicsView._frontGroup.g.matrix
    ) {
      var targetMatrix = lyrView.graphicsView._frontGroup.g.matrix;
      if (_.isArray(targetMatrix)) {
        // 4.6版本 matrix为数组
        targetMatrix = {
          xx: targetMatrix[0],
          yx: targetMatrix[1],
          xy: targetMatrix[2],
          yy: targetMatrix[3],
          dx: targetMatrix[4],
          dy: targetMatrix[5],
        };
      }
      m = matrix.clone(targetMatrix);
    }
    return m;
  }

  toMatrixString(m) {
    return (
      'matrix(' +
      [
        m.xx.toFixed(8),
        m.yx.toFixed(8),
        m.xy.toFixed(8),
        m.yy.toFixed(8),
        m.dx.toFixed(8),
        m.dy.toFixed(8),
      ].join() +
      ')'
    );
  }

  addTipManager() {
    this.openedPopups = [];
    //后期做tipMananger
    // this.tipManager = new TipManager({
    //     view: this.view,
    //     container: this.$el.find(".gis-view-surface")
    // });
    // this.tipManager.render();
  }

  renderRightClickMenu() {}

  renderRoamte() {}

  createLayer(config, collection, returnLayer, layerType) {
    var layer;
    var layers = [];
    if (config.url) {
      config.url = this.loadedModules.urlUtils.getUrl(config.url);
    }
    switch (config.layerType.toLowerCase()) {
      case 'tilelayer':
        layer = new this.loadedModules.TileLayer(config);
        break;
      case 'pipenetlayer':
      case 'mapimagelayer':
        layer = new this.loadedModules.MapImageLayer(
          Utils._.extend({}, config, {
            token: _config.token,
            client: 'webgis',
            visible: false,
          }),
        );
        layer.then(
          function () {
            layer.visible = true;
            if (layer.hasAnnotation) {
              //  layer.annotationLayer = this.addAnnotationLayer(layer.title, layer.url, config.visible);
            }
          }.bind(this),
        );
        break;
      case 'elevationlayer':
        layer = new this.loadedModules.ElevationLayer(config);
        break;
      case 'scenelayer':
        layer = new this.loadedModules.SceneLayer(config);
        break;
      case 'featurelayer':
        layer = new this.loadedModules.FeatureLayer(config);
        break;
      case 'grouplayer':
        var layers = config.layers.map(function (c) {
          return this.createLayer(c, [], true, '');
        }, this);
        layer = new this.loadedModules.GroupLayer(
          Utils._.extend({}, config, {
            layers: layers,
          }),
        );
        break;
      case 'google-v':
      case 'google-i':
      case 'google-i-a':
      case 'google-ia':
        layer = new this.loadedModules.GoogleTileLayer(
          Utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'google-user':
        var proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new this.loadedModules.UserTileLayer(
          Utils._.extend(config, {
            url: proxyUrl,
          }),
        );
        layer.then((res) => console.log(res));
        break;
      case 'pipenet-tile':
        var proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new this.loadedModules.PipenetTileLayer(
          Utils._.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'baidu-v':
      case 'baidu-i':
      case 'baidu-ia':
        layer = new this.loadedModules.BaiduTileLayer(
          Utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'amap-v':
      case 'amap-i':
      case 'amap-ia':
      case 'amap-v2':
      case 'amap-i2':
      case 'amap-ia2':
        layer = new this.loadedModules.AMapTileLayer(
          Utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v':
      case 'tianditu-va':
      case 'tianditu-i':
      case 'tianditu-ia':
      case 'tianditu-t':
      case 'tianditu-ta':
      case 'tianditu-ll-v':
      case 'tianditu-ll-va':
      case 'tianditu-ll-i':
      case 'tianditu-ll-ia':
      case 'tianditu-ll-t':
      case 'tianditu-ll-ta':
        layer = new this.loadedModules.TiandituTileLayer(
          Utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v-va':
        //深拷贝config
        var configString = JSON.stringify(config);
        //天地图矢量
        var configv = Utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-v',
        });
        this.createLayer(configv, collection, false, layerType);

        //天地图注记
        var configva = Utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-va',
        });
        this.createLayer(configva, collection, false, layerType);
        break;
      case 'tianditu-i-ia':
        //深拷贝config
        var configString = JSON.stringify(config);
        //天地图矢量
        var configi = Utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-i',
        });
        this.createLayer(configi, collection, false, layerType);

        //天地图注记
        var configia = Utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-ia',
        });
        this.createLayer(configia, collection, false, layerType);

        break;
      case 'osm':
        layer = new this.loadedModules.OpenStreetMapLayer(
          Utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'wmts-v':
        layer = new this.loadedModules.WMTSTileLayer(Utils._.extend(config, {}));
        break;
      case 'arcgis-streetpurplishblue':
      case 'arcgis-streetwarm':
      case 'arcgis-community':
      case 'arcgis-streetgray':
        layer = new this.loadedModules.TileLayer(
          Utils._.extend(config, {
            url:
              'http://map.geoq.cn/ArcGIS/rest/services/ChinaOnline' +
              config.layerType.replace('arcgis-', '') +
              '/MapServer',
          }),
        );
        break;
      case 'arcgis-tilelayer':
        if (_config.ip.indexOf('222.92') > -1)
          config.url = config.url.replace(/\d+\.\d+\.\d+\.\d+/, window.location.hostname);
        layer = new this.loadedModules.TileLayer({
          url: config.url,
          title: config.title,
          //    id: _.uniqueId('arcgis-tilelayer_')
        });
        break;
      default:
        if (config.layerType.toLowerCase().indexOf('tianditu-') > -1) {
          layer = new this.loadedModules.TiandituTileLayer(
            Utils._.extend(config, {
              url: 'http://fack.com',
            }),
          );
          break;
        }
        break;
    }
    //returnLayer 是否需要返回layer
    if (returnLayer) return layer;

    //layer未给值
    if (_.isUndefined(layer)) return this._layerCreateErrorLog(config);

    //layerType
    switch (layerType) {
      case 'baseMap':
        layer.visible = true;
        break;
      case 'pipeLayer':
        Utils._.extend(layer, {
          preload: true,
        });
        break;
      default:
        break;
    }
    collection.add(layer);
  }

  loadAreaLayer() {
    var url = window.location.protocol + '//webapi.amap.com/maps?';
    this.areaLayerInit = true;
    this.extent = {
      spatialReference: {
        wkid: 4526,
      },
      xmin: 0,
      ymin: 0,
      xmax: 0,
      ymax: 0,
    };
    if (!this.mapsettings.areasettings) return null;
    //  if (this.basemaps && this.basemaps[0] && this.basemaps[0]['id'] == '自定义底图') return this.setExtent()
    var area = this.mapsettings.areasettings.areaName.split(',');
    if (window.navigator.onLine == true && area.length == 1) {
      let timer = setInterval(
        function () {
          if (window.AMap && window.AMap.DistrictSearch) {
            clearInterval(timer);
            this.getAreaLayer();
          }
        }.bind(this),
        500,
      );
    } else {
      // if (this.getPipenetLayer()) var url = this.getPipenetLayer().url;
      // if (url && _config.mapsettings.areasettings.areaName) {
      //     request(url + "/GetMapArea", {
      //         query: {
      //             areaName: _config.mapsettings.areasettings.areaName.split(',')[0],
      //             outSR: webgisPandaTool.getCurrentMapWkid(),
      //         },
      //         cacheBust: true
      //     }).then(function (r) {
      //         if (r.data && r.data.length > 0) {
      //             this.polygon = [], this.edge = [], this.area = [], this.bound = null;
      //             r.data.forEach(function (point) {
      //                 this.polygon.push([point.x, point.y]);
      //             }.bind(this))
      //             this.edge.push(this.polygon);
      //             _config.useCoverMap && this.area.push(this.polygon);
      //             this._setAreaLayer();
      //         } else {
      //             console.log("行政区图层无数据");
      //         }
      //     }.bind(this), function (error) {
      //         console.log("未查询到行政区图层数据！");
      //     }.bind(this))
      // }
    }
  }

  getAreaLayer() {}

  setExtent() {}

  showPipeNetLayer() {}

  getNearestScale(extent) {
    var resolution = extent.width / this.view.width,
      zoom = 0,
      nearestScale = -1,
      diff = Number.POSITIVE_INFINITY;
    if (this.view.map.basemap && this.view.map.basemap.baseLayers.length > 0) {
      this.view.map.basemap.baseLayers.getItemAt(0).tileInfo.lods.forEach(
        function (item) {
          if (item.resolution >= resolution) {
            var difftemp = item.resolution - resolution;
            if (difftemp < diff) {
              diff = difftemp;
              nearestScale = item.scale;
              zoom = item.level;
            }
          }
        }.bind(this),
      );
    }
    if (!this.mapsettings.scale) {
      this.mapsettings.scale = nearestScale;
      this.mapsettings.zoom = zoom;
    }
    return nearestScale;
  }

  render() {
    return <div ref={this.mapRef} id={this.props.id} style={this.getStyle()}></div>;
  }
}

MapManager.propTypes = {
  id: PropTypes.string,
  width: PropTypes.string,
  height: PropTypes.string,
  mapsettings: PropTypes.object,
  modules: PropTypes.array,
};
