import gisConfig from '../config/gis';
import esriLoader from 'esri-loader';

export const _ = require('lodash');

export function loadModules(modules = [], mapSettings = {}) {
  let mergeMapSettings = Object.assign(gisConfig, mapSettings);
  return new Promise(
    function (resolve, reject) {
      esriLoader.loadModules(modules, mergeMapSettings).then(
        function (array) {
          resolve(array);
        }.bind(this),
      );
    }.bind(this),
  );
}

export async function asyncLoadModules(modules = [], mapSettings = {}) {
  let mergeMapSettings = Object.assign(gisConfig, mapSettings);
  let results = await esriLoader.loadModules(modules, mergeMapSettings);
  return results;
}

export function bindUrlUtils(urlUtils) {
  if (!urlUtils.getUrl) {
    urlUtils.getUrl = urlUtilsGetUrl;
  }
}

function urlUtilsGetUrl(template, _module) {
  var urlArr = [];
  urlArr.push(location.protocol + '/');
  urlArr.push(location.host);
  if (_config.prefix) {
    urlArr.push(_config.prefix);
  }

  if (_module && _module.length) {
    // url服务接口 模块配置路径采用绝对路径
    var _urlModuleTempl = {
      CaseManageREST: 'CityInterface/Services/CityServer_CaseManage/REST/CaseManageREST.svc',
      EditREST: 'CityInterface/Services/CityServer_Edit/REST/EditREST.svc',
      MapREST: 'CityInterface/rest/services/MapServer.svc',
      MapRESTOld: 'CityInterface/Services/CityServer_Map/REST/MapREST.svc',
      WorkFlowREST: 'CityInterface/Services/CityServer_WorkFlow/REST/WorkFlowREST.svc',
      AuditREST: 'CityInterface/Services/CityServer_Audit/REST/AuditREST.svc',
      LocatorREST: 'CityInterface/Services/CityServer_Locator/REST/LocatorREST.svc',
      NewNotificationREST:
        'CityInterface/Services/CityServer_NewNotification/REST/NewNotificationREST.svc',
      OfficeTemplateREST:
        'Cityinterface/Services/CityServer_OfficeTemplate/REST/OfficeTemplate.svc',
      OfflineTaskEditREST:
        'CityInterface/Services/CityServer_OfflineTaskEdit/REST/OfflineTaskEditREST.svc',
      PatrolStandardRest: 'CityInterface/rest/services/PatrolStandard.svc',
      PatrolStandardRestOld:
        'CityInterface/Services/CityServer_Patrol_Standard/REST/PatrolStandardRest.svc',
      NewPatrolREST: 'CityInterface/rest/services/NewPatrol.svc',
      OldPatrolREST: 'CityInterface/Services/CityServer_NewPatrol/REST/Newpatrolrest.svc',
      MobileBusinessBaseREST: 'CityInterface/Services/CityServer_MobileBusiness/REST/BaseREST.svc',
      HZGSProjectBoxREST:
        'CityInterface/Services/CityServer_HZGSProjectBox/REST/HZGSProjectBoxREST.svc',
      ZS_ProjectManageREST:
        'CityInterface/Services/CityServer_ProjectManage_ZS/REST/ProjectManageREST.svc',
    };
    var _templ = _urlModuleTempl[_module] ? _urlModuleTempl[_module] : '';
    urlArr.push(_templ);
    return urlArr.join('/');
  } else {
    if (
      ['{host}', '{protocol}', '{prefix}', '{ip}'].some(function (str) {
        return template.toLowerCase().indexOf(str) >= 0;
      })
    ) {
      return template
        .replace(/\{host\}/gi, urlArr.join('/'))
        .replace(/\{protocol\}/gi, _config.protocol + '//')
        .replace(/\{prefix\}/gi, _config.prefix || '')
        .replace(/\{ip\}/gi, _config.ip);
    } else if (template.indexOf('http://') == 0 || template.indexOf('https://') == 0) {
      return template;
    } else {
      urlArr.push(template);
      return urlArr.join('/');
    }
  }
}
